// ********************************************************************
// Projekt Definitionen
// AVR-Version / GGC 4.3.2
// Version 1.0 / Stand: 25.06.2010
// Autor: Matthias Kahnt
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Autoren erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// Portverwendung
//************************************************************************
// VDIP1: D[0..7]    PA[0..7]       I/O (SPI: D0=SCK, D1=SDI, D2=SDO, D3=CS)
//        RD#        PC6            O
//        WR         PC7            O
//        RXF#       PC4            I
//        TXE#       PC5            I
//        RES        PC2            O
// LCD:   MOSI (RW)  PB5 (MOSI)     O
//        MISO (D0)  PB6 (MISO)     I
//        SCK  (E)   PB7 (SCK)      O
//        RES        PC3            O
//        CS   (RS)  PC1            O
// LED:   MOSI (SER) PB5 (MOSI)     O
//        SCK  (SCK) PB7 (SCK)      O
//        ENL  (RCK) PB4            O
// KEYS:  ESC        PB3            I
//        SET-       PB2            I
//        SET+       PB1            I
//        ET         PB0            I
// J1:    JUM        PC0            I
// UART:  RXD (2)    PD2 (RXD1)     I
//        TXD (3)    PD0 (RXD0)     I
//        DTR (4)    PD6            I
//        DSR (6)    PD7            I
//        RTS (7)    PD4            I
//        CTS (8)    PD5            I
//        RI  (9)    PD3            I
//        POL        PD1            I

//************************************************************************
// Definebereich
//************************************************************************

//************************************************************************
// AVR-Typ: ATmega1284P
#define F_CPU   18432000UL		/* AVR FREQUENZ */
//************************************************************************

#define PX7 7                   /* PORT-PINS (A..D) */
#define PX6 6
#define PX5 5
#define PX4 4
#define PX3 3
#define PX2 2
#define PX1 1
#define PX0 0

#define SPALTEN_MAX     20      /* LCD: Zeichen je Zeile                        */
#define ZEILEN_MAX      4       /* LCD: Anzahl der Zeilen                       */
#define SCOLL_DELAY     50      /* Pause zwischen 2 Zeichen in [us]             */

#define HGL_ZEIT        30000   /* Zeit fr LCD Hintergrund-Beleuchtung in [ms] */
#define MENUE_TIMEOUT   50000   /* Zeit bis automatischen Men-Abbruch in [ms]  */
#define TASTEN_REP_BEG  2000    /* Zeit bis zur 1. Tastenwiederholung in [ms]   */
#define TASTEN_REP_ZT   250     /* Zeit der Tastewiederholung in [ms]           */
#define TASTEN_NOREPEAT 0       /* keine Tastenwiederholung                     */
#define TASTEN_REPEAT   1       /* Tastenwiederholung                           */

#define ESC             1       /* Tasten-Kodierungen                           */
#define SETM            2
#define SETP            3
#define ET              4
#define KEY_TIMEOUT     5

// VNC1L SPI-Interface
#define SCLK_L          PORTA &= ~(1<<PX0)  /* SCLK VNC1L */
#define SCLK_H          PORTA |=  (1<<PX0)

#define SDI_L           PORTA &= ~(1<<PX1)  /* SDI  VNC1L */
#define SDI_H           PORTA |=  (1<<PX1)

#define SDO_VNC         (PINA & (1<<PX2))   /* SDO  VNC1L */

#define CS_VNC_L        PORTA &= ~(1<<PX3)  /* CS   VNC1L */
#define CS_VNC_H        PORTA |=  (1<<PX3)

#define RE_VNC_L        PORTC &= ~(1<<PX2)  /* RS   VNC1L */
#define RE_VNC_H        PORTC |=  (1<<PX2)

#define PROMPT_CHECK            2   /* VDIP1/VNC1L Konstanten   */
#define PROMPT_UND_ERROR_CHECK  1
#define PROMPT_OK               1
#define FILE_NEW                0
#define FILE_APPEND             1
#define TIMEOUT_SPI             50  /* Anzahl der SPI Schreib-/Leseversuche                 */
#define TIMEOUT_READ_WRITE      500 /* Anzahl der Read-/Writeversuche falls Buffer voll     */
#define TIMEOUT_KOMMUNIKATION   5   /* Timeout falls keine Kommunikation zu Stande kommt    */

// VNC1L FIFO-Interface (z. Zt. nicht verwendet!)
#define RES_VDIP1_L     PORTC &= ~(1<<PX2)  /* RESET VDIP1 */
#define RES_VDIP1_H     PORTC |=  (1<<PX2)

#define WR_VDIP1_L      PORTC &= ~(1<<PX7)  /* WR VDIP1    */
#define WR_VDIP1_H      PORTC |=  (1<<PX7)

#define RD_VDIP1_L      PORTC &= ~(1<<PX6)  /* RD# VDIP1   */
#define RD_VDIP1_H      PORTC |=  (1<<PX6)

#define RXF_VDIP1       (PINC & (1<<PX4))   /* RXF# VDIP1  */
#define TXE_VDIP1       (PINC & (1<<PX5))   /* TXE# VDIP1  */

#define DATA_VDIP1_WR   PORTA               /* DATA schreiben VDIP1     */
#define DATA_VDIP1_RD   PINA                /* DATA lesen VDIP1         */

#define DATA_VDIP1_PORT DDRA                /* DATA Port-Richtung VDIP1 */

// LCD-Interface
#define RES_LCD_L       PORTC &= ~(1<<PX3)  /* RESET LCD   */
#define RES_LCD_H       PORTC |=  (1<<PX3)

#define CS_LCD_L        PORTC &= ~(1<<PX1)  /* ENABLE LCD  */
#define CS_LCD_H        PORTC |=  (1<<PX1)

// LED Schieberegister-Interface
#define EN_595_L        PORTB &= ~(1<<PX4)  /* ENABLE 595  */
#define EN_595_H        PORTB |=  (1<<PX4)

// Tastatur
#define TASTE_ESC       (PINB & (1<<PX3))   /* Taste ESC   */
#define TASTE_SETM      (PINB & (1<<PX2))   /* Taste SET-  */
#define TASTE_SETP      (PINB & (1<<PX1))   /* Taste SET+  */
#define TASTE_ET        (PINB & (1<<PX0))   /* Taste ET    */

// UART Status
#define DTR_RS232       (PIND & (1<<PX6))   /* DTR UART    */
#define DSR_RS232       (PIND & (1<<PX7))   /* DSR UART    */
#define RTS_RS232       (PIND & (1<<PX4))   /* RTS UART    */
#define CTS_RS232       (PIND & (1<<PX5))   /* CTS UART    */
#define RIN_RS232       (PIND & (1<<PX3))   /* RI  UART    */

// UART Polaritten fr Pin2 und Pin3
#define POL_RS232_L     PORTD &= ~(1<<PX1)  /* POL UART    */
#define POL_RS232_H     PORTD |=  (1<<PX1)

// LEDs
#define LED_D1_EIN      ledimage &= 0b01111111
#define LED_D1_AUS      ledimage |= 0b10000000

#define LED_D2_EIN      ledimage &= 0b10111111
#define LED_D2_AUS      ledimage |= 0b01000000

#define LED_RTS_EIN     ledimage &= 0b11011111
#define LED_RTS_AUS     ledimage |= 0b00100000

#define LED_CTS_EIN     ledimage &= 0b11101111
#define LED_CTS_AUS     ledimage |= 0b00010000

#define LED_DTR_EIN     ledimage &= 0b11110111
#define LED_DTR_AUS     ledimage |= 0b00001000

#define LED_DSR_EIN     ledimage &= 0b11111011
#define LED_DSR_AUS     ledimage |= 0b00000100

#define LED_RI_EIN      ledimage &= 0b11111101
#define LED_RI_AUS      ledimage |= 0b00000010

#define LCD_HGBL_AUS    ledimage &= 0b11111110
#define LCD_HGBL_EIN    ledimage |= 0b00000001

#define LED_LCD_AUS     ledimage  = 0xFE

// Jumper
#define JUMPER_JP1      (PINC & (1<<PX0))  /* Jumper JP1 */


//************************************************************************
// Gemeinsame Strukturen
//************************************************************************

// Struktur der Anwender- und Systemeinstellungen (EEPROM)
struct awd_struktur {
  volatile unsigned char eeprom_code;        // Kennbyte fr Daten im EEPROM gltig
  volatile unsigned char uart0_baudrate;     // UART0 (PIN3) Baudrate (0=150 1=300, 2=600, 3=1200, 4=2400, 5=4800, 6=9600, 7=14400 8=19200 9=38400 10=57600, 11=76800, 12=115200, 13=230400)
  volatile unsigned char uart0_parity;       // UART0 (PIN3) Paritt (0=keine, 1=Even, 2=Odd)
  volatile unsigned char uart0_datenbits;    // UART0 (PIN3) Datenbits (0=5, 1=6, 2=7, 3=8, 4=9)
  volatile unsigned char uart1_baudrate;     // UART1 (PIN2) Baudrate (0=150 1=300, 2=600, 3=1200, 4=2400, 5=4800, 6=9600, 7=14400 8=19200 9=38400 10=57600, 11=76800, 12=115200, 13=230400)
  volatile unsigned char uart1_parity;       // UART1 (PIN2) Paritt (0=keine, 1=Even, 2=Odd)
  volatile unsigned char uart1_datenbits;    // UART1 (PIN2) Datenbits (0=5, 1=6, 2=7, 3=8, 4=9)
  volatile unsigned char uart01_pol;         // UART0/1 Pegelinvertierung (1=invertieren, 0=unverndert)
  volatile unsigned char uart_log_mode;      // Log-Mode (0=Mode0, 1=Mode1 ... Mode5)
  volatile unsigned char uart_app_mode;      // Log-Verhalten (1=Anhngen, 0=New)
  volatile unsigned char stick_enable;       // USB-Stick Aufzeichnung (0=nein, 1=ja )
  volatile unsigned char stick_dateiformat;  // USB-Stick Dateiformat (0=txt/csv,1=bin)
  volatile unsigned char stick_speed;        // USB-Stick Timing (0=schnell,1=langsam)
  volatile unsigned char lcd_hgbl;           // LCD Hintergrundbeleuchtung (0=aus, 1=ein, 2=auto)
  volatile unsigned char software_version;   // Firmwarestand
};

// Struktur der Menpunkte
struct parameter_struktur {
  unsigned char  nr;                // Parameternummer
  unsigned char  defa;              // Defaultwert
  unsigned char  max;               // Maximaler des Parameterwertes
  unsigned char  min;               // Mimimaler des Parameterwertes
  unsigned char  delta;             // Inc/Dec-Wert
  unsigned char  funktion;          // Auszufhrende Funktion
  unsigned char  aenderung;         // nderungsmglichkeit 1=nderbar 0=nicht nderbar
  char txt[25];                     // Anzeigetext des Parameters
  volatile unsigned char*  adresse; // Adresse der dazugehrigen Variable
};

// Struktur der gelogten Daten (Logbuch)
#define LOGBUCH0_MAX    1800    /*  2000 0 ... 1999 */
#define LOGBUCH1_MAX    4200    /*  4600 0 ... 4599 */
#define LOGBUCH2_MAX    2100    /*  2300 0 ... 2299 */
#define LOGBUCH3_MAX    6300    /*  7000 0 ... 6999 */
#define LOGBUCH4_MAX   12600    /* 14000 0 ...13999 */
#define LOGBUCH5_MAX   12600    /* 14000 0 ...13999 */

// Mode 0: PIN2, PIN3, Status, Timecode
// Mode 1: PIN2, PIN3, Status
// Mode 2: PIN2, PIN3, Timecode
// Mode 3: PIN2, PIN3
// Mode 4: PIN2
// Mode 5: PIN3

union log_union {
    struct log_mode0 { // belegt 7 Byte
        volatile unsigned char     rxd_pin2;
        volatile unsigned char     rxd_pin3;
        volatile unsigned char     status;
        volatile unsigned long     timecode;
    } mode0[LOGBUCH0_MAX];

    struct log_mode1 { // belegt 3 Byte
        volatile unsigned char     rxd_pin2;
        volatile unsigned char     rxd_pin3;
        volatile unsigned char     status;
    } mode1[LOGBUCH1_MAX];

    struct log_mode2 { // belegt 6 Byte
        volatile unsigned char     rxd_pin2;
        volatile unsigned char     rxd_pin3;
        volatile unsigned long     timecode;
    } mode2[LOGBUCH2_MAX];

    struct log_mode3 { // belegt 2 Byte
        volatile unsigned char     rxd_pin2;
        volatile unsigned char     rxd_pin3;
    } mode3[LOGBUCH3_MAX];

    struct log_mode4 { // belegt 1 Byte
        volatile unsigned char     rxd_pin2;
    } mode4[LOGBUCH4_MAX];

    struct log_mode5 { // belegt 1 Byte
        volatile unsigned char     rxd_pin3;
    } mode5[LOGBUCH5_MAX];
};

// Log-Struktur
struct log_struc {
    volatile unsigned char enable;       // Freigabe Logging
    volatile unsigned int  index;        // aktuelle Speichertiefe
    volatile unsigned int  max;          // max. mgliche Speichertiefe
};

// LCD-Struktur
struct lcd_struc {
    unsigned char x;                    // Spalte 
    unsigned char y;                    // Zeile
    unsigned int scrolldelay;           // Pausenzeit [ms] Zeilenscroll fr die Monitorfunktion
    char buffer[30];                    // Buffer fr Vorformatierungen zur LCD-Ausgabe
};
//#define LCD_MONITORING                /* aus- / einkommentieren fr TEST-Zwecke */

// VNC1L Struktur
#define VNC_BUFFER_SIZE 70              /* Sende-/Empfangsbuffer Gre mindestens 70 */
struct vnc_struc {
    char  rec_buffer[VNC_BUFFER_SIZE];  // Empfangsbuffer letzte Daten
    char send_buffer[VNC_BUFFER_SIZE];  // Sendebuffer
    char temp_buffer[20];               // Kleiner Hilfs-Sendebuffer
    unsigned char file_index;           // Zhler fr Dateinamen (00 - 99)
    unsigned char status;               // Status
};
