// ********************************************************************
// SPI - Grundfunktionen 
// AVR-Version / GGC 4.3.2
// Version 1.0 / Stand: 20.10.2010
// Autor: Matthias Kahnt
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// des Autors erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// AVR/GCC LIBS
//************************************************************************
#include "defines.h"            // Projektdefinitionen
#include <avr/io.h>             // IO-Grundfunktionen
#include <stdio.h>              // Standardfunktion
#include "spi_avr.h"            // Headerfile

// ***********************************************************************
// SPI - Initialisierung
// ***********************************************************************
void spi_master_init(void) {
  DDRB  |=  ((1<<PX5) | (1<<PX7)) ; // MOSI(PB5), SCK(PB7) (Ausgnge)
  DDRB  &=  ~(1<<PX6);              // MISO(PB6) (Eingang)
  PORTB |=   (1<<PX6);              // MISO(PB6) Pull-Up-R zuschalten
  // Grundinitialisierung (SPI-Freigabe, Master, LSB-First, Mode 3, Fosz/64)
  SPCR   =  ((1<<SPE) | (1<<MSTR) | (1<<DORD) | (1<<SPR0) | (1<<CPOL) | (1<<CPHA));
}

// ***********************************************************************
// SPI - Datenbyte senden
// ***********************************************************************
void spi_master_transmit(unsigned char datenbyte) {
  SPDR = datenbyte;                 // Schreiben der Daten
  while (!(SPSR & (1<<SPIF)));
}

// ***********************************************************************
// SPI - Datenbyte empfangen
// ***********************************************************************
unsigned char spi_master_receive(void) {
  SPDR = 0x00;                      // Dummy-Byte senden
  while (!(SPSR & (1<<SPIF)));
  return SPDR;                      // empfangenes Byte zurckgeben
}

// ***********************************************************************
// HC595 - LED ein-/ausschalten
// ***********************************************************************
void led_set(unsigned char ledimage) {
  spi_master_transmit(ledimage);
  EN_595_H;
  EN_595_L;
}

